@@ -11,10 +11,13 @@
 #include <util.h>
 #include <cipher.h>
 
+#include <gtkaccount.h>
+
 #include "hippo-dbus-helper.h"
 #include <dbus/dbus-glib-lowlevel.h>
 
 #define HIPPO_DBUS_IM_INTERFACE "org.freedesktop.od.IM"
+#define HIPPO_DBUS_IMCLIENT_INTERFACE "org.freedesktop.od.IMClient"
 #define HIPPO_DBUS_IM_PATH "/org/freedesktop/od/im"
 
 typedef struct {
@@ -342,6 +345,21 @@ static const HippoDBusMember im_members[] = {
     { 0, NULL }
 };
 
+static DBusMessage*
+handle_configure(void            *object,
+                 DBusMessage     *message,
+                 DBusError       *error)
+{
+    pidgin_accounts_window_show();
+
+    return dbus_message_new_method_return(message);
+}
+
+static const HippoDBusMember imclient_members[] = {
+    { HIPPO_DBUS_MEMBER_METHOD, "Configure", "", "", handle_configure },
+    { 0, NULL }
+};
+
 static void
 emit_buddy_list_changed (DBusConnection *connection)
 {
@@ -486,9 +504,13 @@ plugin_load(PurplePlugin *plugin)
     hippo_dbus_helper_register_interface(connection, HIPPO_DBUS_IM_INTERFACE,
                                          im_members, NULL);
     
+    hippo_dbus_helper_register_interface(connection, HIPPO_DBUS_IMCLIENT_INTERFACE,
+                                         imclient_members, NULL);
+
     hippo_dbus_helper_register_object(connection, HIPPO_DBUS_IM_PATH,
                                       pd,
-                                      HIPPO_DBUS_IM_INTERFACE, NULL);
+                                      HIPPO_DBUS_IM_INTERFACE,
+                                      HIPPO_DBUS_IMCLIENT_INTERFACE, NULL);
 
     purple_signal_connect(purple_blist_get_handle(), "buddy-added",
                           get_signal_handle(),
