@@ -1,12 +1,13 @@
 import sys, logging, urlparse, gnome
 
-import gobject, gtk, dbus
+import gobject, gtk, dbus, pango
 
-import gutil
-from ddm import DataModel
+import gutil, ddm_util
 
 _logger = logging.getLogger("weblogindriver.AccountsDialog")
 
+DIALOG_WIDTH = 320
+
 class Dialog(gtk.Window):      
     def __init__(self, account_types, online_accounts_service, *args, **kwargs):
         super(Dialog, self).__init__(*args, **kwargs)        
@@ -25,6 +26,7 @@ class Dialog(gtk.Window):
         self.set_position(gtk.WIN_POS_CENTER)
 
         self.__notebook = gtk.Notebook()
+        self.__notebook.set_size_request(DIALOG_WIDTH, -1)
         self.__notebook.set_border_width(5)
 
         self.__outer_existing_accounts_vbox = gtk.VBox() 
@@ -57,9 +59,16 @@ class Dialog(gtk.Window):
         # self.__existing_accounts_vbox.pack_start(self.__no_accounts_label)
 
         self.__model = gtk.ListStore(gobject.TYPE_STRING)
+        self.__model.set_sort_column_id(0, gtk.SORT_ASCENDING)
         self.__accounts_combo = gtk.ComboBox(self.__model)
         self.__existing_accounts_vbox.pack_start(self.__accounts_combo, True, False)
         textrender = gtk.CellRendererText()
+        # If we set ellipsize mode, the text gets ellipsized both in the selection pop-out and
+        # and in the selected item view. It would be ideal to have it be full length in the 
+        # first case, but not in the second case. Otherwise, having it show up full length in
+        # the selection pop-out, and having it truncated instead of ellipsized in the selected
+        # item view doesn't look so bad.
+        # textrender.set_property("ellipsize", pango.ELLIPSIZE_MIDDLE)
         self.__accounts_combo.pack_start(textrender, True)
         self.__accounts_combo.add_attribute(textrender, 'text', 0)
         self.__accounts_combo.connect('notify::active', self.__on_edited_account_changed)
@@ -107,45 +116,56 @@ class Dialog(gtk.Window):
         remove_image = gtk.Image()
         remove_image.set_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_BUTTON)
         self.__remove_button.set_image(remove_image) 
-        self.__remove_button.set_sensitive(False)
+        # self.__remove_button.set_sensitive(False)
         self.__remove_button.connect('clicked',
                                    self.__on_account_remove_clicked)
         remove_button_box.pack_end(self.__remove_button, expand=False, fill=False)
-
-        self.__remove_link_box = gtk.HBox()
-        self.__remove_link = Link()
-        self.__remove_link.set_text("Remove account online")
-        self.__remove_link.set_enabled(False)
-        self.__remove_link.connect("clicked", self.__open_account_page)
-        self.__remove_link_box.pack_end(self.__remove_link, expand=False, fill=False)
-
         self.__existing_accounts_vbox.pack_start(remove_button_box, False, False, padding=5)
-        self.__existing_accounts_vbox.pack_start(self.__remove_link_box, False, False)   
+
+        online_options_hbox = gtk.HBox(spacing=2)
+        self.__save_online_check_box = gtk.CheckButton(label="Save Accounts Online")
+        self.__save_online_check_box.set_active(self.__online_accounts_service.GetSaveOnlineFlag())
+        self.__save_online_check_box.connect('toggled',
+                                             self.__on_save_online_setting_changed)
+        online_options_hbox.pack_start(self.__save_online_check_box, False, False)  
+
+        self.__account_page_link_box = gtk.HBox()
+        self.__account_page_link = Link()
+        self.__account_page_link_connect_id = None
+        self.__account_page_link_box.pack_start(self.__account_page_link, expand=False, fill=False)
+    
+        self.__outer_existing_accounts_vbox.pack_end(self.__account_page_link_box, False, False, padding=2) 
+        self.__outer_existing_accounts_vbox.pack_end(online_options_hbox, False, False)   
+        separator = gtk.HSeparator()
+        self.__outer_existing_accounts_vbox.pack_end(separator, False, False)   
 
         # stuff below gets added to the "Add Account" tab
 
         self.__account_types_model = gtk.ListStore(gobject.TYPE_STRING)
+        self.__account_types_model.set_sort_column_id(0, gtk.SORT_ASCENDING)
         self.__account_types_combo = gtk.ComboBox(self.__account_types_model)
         self.__new_account_vbox.pack_start(self.__account_types_combo, True, False)
         account_types_textrender = gtk.CellRendererText()
         self.__account_types_combo.pack_start(account_types_textrender, True)
         self.__account_types_combo.add_attribute(account_types_textrender, 'text', 0)
-        
+
         _logger.debug("will get accounts types")
         self.__all_account_types = self.__online_accounts_service.GetAllAccountTypes()
         _logger.debug("will get accounts types 2")
+        self.__user_info_types_by_full_name = {}
+        self.__type_links_by_full_name = {}
      
-        # TODO: once we save accounts added locally on server, we will allow adding all account types
-        # in the dialog, but for now we don't allow adding any
-        # for type_full_name in self.__all_account_types.values():
-        #     self.__account_types_model.append([type_full_name])
-            
+        for (account_type, (type_full_name, user_info_type, type_link)) in self.__all_account_types.items():
+            if (not self.__account_types) or account_type in self.__account_types:
+                self.__account_types_model.append([type_full_name])
+                self.__user_info_types_by_full_name[type_full_name] = user_info_type 
+                self.__type_links_by_full_name[type_full_name] = type_link          
+  
         # Setting padding to 1 here is a hack to get the content of both tabs to be aligned,
         # we'll need to change this when it will become possible to add new account types to
         # the existing dialog, for example, by adding stocks that require these account types
         padding = 1  
         if len(self.__account_types_model) > 0:
-            self.__account_types_combo.set_active(0) 
             padding = 5
 
         self.__username_entry = gtk.Entry()
@@ -154,12 +174,15 @@ class Dialog(gtk.Window):
         self.__username_entry.connect('activate',
                                       self.__on_new_username_added)
 
-        username_vbox = gtk.VBox(spacing=2)
-        self.__username_label = gtk.Label("Username:")
-        self.__username_label.set_alignment(0.0, 0.5)
-        username_vbox.pack_start(self.__username_label, False, False)    
-        username_vbox.pack_end(self.__username_entry, False, False)
-        self.__new_account_vbox.pack_start(username_vbox, padding=padding)
+        self.__username_vbox = gtk.VBox(spacing=2)
+        self.__username_link = Link()
+        self.__username_link.set_size_request(DIALOG_WIDTH - 20, -1)
+        self.__username_link.set_alignment(0.0, 0.5)
+        self.__username_link.set_line_wrap(True)  
+        self.__username_link_connect_id = None  
+        self.__username_vbox.pack_start(self.__username_link, False, False)  
+        self.__username_vbox.pack_end(self.__username_entry, False, False)
+        self.__new_account_vbox.pack_start(self.__username_vbox, padding=padding)
 
         self.__add_button_box = gtk.HBox()
         self.__add_button = gtk.Button(label="Add Account")
@@ -170,34 +193,43 @@ class Dialog(gtk.Window):
         self.__add_button.connect('clicked',
                                    self.__on_new_username_added)
         self.__add_button_box.pack_end(self.__add_button, expand=False, fill=False)
-        self.__new_account_vbox.pack_start(self.__add_button_box, False, False, padding=padding)
+        self.__new_account_vbox.pack_start(self.__add_button_box, False, False, padding=padding)   
+
+        self.__account_types_combo.connect('notify::active', self.__on_edited_account_type_changed)
+        if len(self.__account_types_model) > 0:
+            self.__account_types_combo.set_active(0) 
+
+        online_options_hbox_2 = gtk.HBox(spacing=2)
+        self.__save_online_check_box_2 = gtk.CheckButton(label="Save Accounts Online")
+        self.__save_online_check_box_2.set_active(self.__online_accounts_service.GetSaveOnlineFlag())
+        self.__save_online_check_box_2.connect('toggled',
+                                               self.__on_save_online_setting_changed)
+        online_options_hbox_2.pack_start(self.__save_online_check_box_2, False, False)  
+
+        self.__account_page_link_box_2 = gtk.HBox()
+        self.__account_page_link_2 = Link()
+        self.__account_page_link_2_connect_id = None
+        self.__account_page_link_box_2.pack_start(self.__account_page_link_2, expand=False, fill=False)
+    
+        self.__outer_new_account_vbox.pack_end(self.__account_page_link_box_2, False, False, padding=2) 
+        self.__outer_new_account_vbox.pack_end(online_options_hbox_2, False, False)   
+        separator_2 = gtk.HSeparator()
+        self.__outer_new_account_vbox.pack_end(separator_2, False, False)  
 
-        self.__add_link_box = gtk.HBox()
-        self.__add_link = Link()
-        self.__add_link.connect("clicked", self.__open_account_page)
         if len(self.__account_types_model) > 0:
             self.__account_types_combo.set_active(0)
-            self.__add_link.set_text("Add other accounts online")
-            self.__add_link_box.pack_end(self.__add_link, expand=False, fill=False)
-            self.__outer_new_account_vbox.pack_end(self.__add_link_box, False, False)
-        else:  
-            if self.__account_types and len(self.__account_types) == 1 and \
-               self.__account_types[0] in self.__all_account_types.keys():           
-                self.__add_link.set_text("Add " + self.__all_account_types[self.__account_types[0]] + " accounts online")
-            else:
-                self.__add_link.set_text("Add accounts online")
-            self.__add_link_box.pack_start(self.__add_link)
-            self.__new_account_vbox.pack_start(self.__add_link_box)
    
         self.__model_tree_iter_by_account = {}
         self.__current_account = None
 
+        self.__update_online_links()        
+
         self.show_all()
         self.__set_no_accounts_state(True)
         
         if len(self.__account_types_model) == 0:
             self.__account_types_combo.hide()
-            self.__username_label.hide()
+            self.__username_link.hide()
             self.__username_entry.hide()
             self.__add_button.hide()
  
@@ -220,23 +252,86 @@ class Dialog(gtk.Window):
         id = self.__onlineaccounts_proxy.connect_to_signal('AccountRemoved',
                                                            self.__on_account_removed)
         self.__connections.add(self.__onlineaccounts_proxy, id)
+        id = self.__onlineaccounts_proxy.connect_to_signal('SaveOnlineFlagChanged',
+                                                           self.__on_save_online_flag_changed)
+        self.__connections.add(self.__onlineaccounts_proxy, id)        
+        id = self.__onlineaccounts_proxy.connect_to_signal('HaveOnlineUserFlagChanged',
+                                                           self.__on_have_online_user_flag_changed)
+        self.__connections.add(self.__onlineaccounts_proxy, id)  
+        id = self.__onlineaccounts_proxy.connect_to_signal('ConnectedToServerFlagChanged',
+                                                           self.__on_connected_to_server_flag_changed)
+        self.__connections.add(self.__onlineaccounts_proxy, id)  
+        id = self.__onlineaccounts_proxy.connect_to_signal('AccountUpdateFeedback',
+                                                           self.__on_account_update_feedback)
+        self.__connections.add(self.__onlineaccounts_proxy, id)        
+
+    def __open_server_page(self, l, page):
+        gnome.url_show(urlparse.urljoin(ddm_util.get_baseurl(), page))
+
+    def __open_page(self, l, page):
+        gnome.url_show(page)
+
+    def __update_online_links(self):
+        save_online_flag = self.__online_accounts_service.GetSaveOnlineFlag()
+        have_online_user_flag = self.__online_accounts_service.GetHaveOnlineUserFlag()  
+        connected_to_server_flag = self.__online_accounts_service.GetConnectedToServerFlag()  
+
+        if self.__account_page_link_connect_id is not None:
+            self.__account_page_link.disconnect(self.__account_page_link_connect_id)
+
+        if self.__account_page_link_2_connect_id is not None:
+            self.__account_page_link_2.disconnect(self.__account_page_link_2_connect_id)
+
+        if save_online_flag and have_online_user_flag and connected_to_server_flag:
+            # display a link to the account page
+            self.__account_page_link.set_text("Visit Account Page")
+            self.__account_page_link_connect_id = self.__account_page_link.connect("clicked", self.__open_server_page, "/account")
+            self.__account_page_link_2.set_text("Visit Account Page")
+            self.__account_page_link_2_connect_id = self.__account_page_link_2.connect("clicked", self.__open_server_page, "/account")
+        elif save_online_flag and not have_online_user_flag:
+            # tell the user that they need to sign in
+            # connected_to_server_flag will always be False when we don't have a user, so we need to check this case first
+            # we can't tell if the server is actually available or not in the case we don't have a user, but that's ok  
+            self.__account_page_link.set_text("Click Here to Sign in", "red")
+            self.__account_page_link_connect_id = self.__account_page_link.connect("clicked", self.__open_server_page, "/who-are-you")
+            self.__account_page_link_2.set_text("Click Here to Sign in", "red")
+            self.__account_page_link_2_connect_id = self.__account_page_link_2.connect("clicked", self.__open_server_page, "/who-are-you")
+        elif save_online_flag and not connected_to_server_flag:
+            # tell the user that the server is not available (either it's down or they are not online)
+            self.__account_page_link.set_text("The Server is Not Available")
+            self.__account_page_link_connect_id = self.__account_page_link.connect("clicked", self.__open_server_page, "")
+            self.__account_page_link_2.set_text("The Server is Not Available")
+            self.__account_page_link_2_connect_id = self.__account_page_link_2.connect("clicked", self.__open_server_page, "")
+        else:
+            # display a Learn More link 
+            self.__account_page_link.set_text("Learn More")
+            self.__account_page_link_connect_id = self.__account_page_link.connect("clicked", self.__open_server_page, "/accounts-learnmore")
+            self.__account_page_link_2.set_text("Learn More")
+            self.__account_page_link_2_connect_id = self.__account_page_link_2.connect("clicked", self.__open_server_page, "/accounts-learnmore") 
+
+    def __on_save_online_flag_changed(self, new_save_online_flag):
+        self.__save_online_check_box.set_active(new_save_online_flag)
+        self.__save_online_check_box_2.set_active(new_save_online_flag)
+        self.__update_online_links()
+
+    def __on_have_online_user_flag_changed(self, new_have_online_user_flag):
+        self.__update_online_links() 
+
+    def __on_connected_to_server_flag_changed(self, new_connected_to_server_flag):
+        self.__update_online_links() 
 
-    def __open_account_page(self, l):
-        gnome.url_show(urlparse.urljoin(get_baseurl(), "/account"))
-      
     def __on_account_added(self, a_path):
-        # TODO: check for account type when we have a per-account dialog
         if a_path not in self.__model_tree_iter_by_account:
             a = self.__online_accounts_service.get_existing_account(a_path)
             if a is None:
                 _logger.error("onlineaccount for path %s was not found" % a_path)
                 return
 
-            if self.__account_types and a.GetType() not in self.__account_types:
+            if (self.__account_types and a.GetType() not in self.__account_types) or a.GetType() not in self.__all_account_types:
                 return 
 
             _logger.debug("account added %s" % a) 
-            tree_iter = self.__model.append([self.__all_account_types[a.GetType()] + ': ' + a.GetUsername()])
+            tree_iter = self.__model.append([self.__all_account_types[a.GetType()][0] + ': ' + a.GetUsername()])
                 
             self.__model_tree_iter_by_account[a_path] = tree_iter
 
@@ -276,7 +371,7 @@ class Dialog(gtk.Window):
             self.__undo_button.hide()
             self.__apply_button.hide()
             self.__remove_button.hide()
-            self.__remove_link_box.hide()
+            # self.__account_page_link_box.hide()
             self.__existing_accounts_label.set_text("No existing accounts") 
             self.__existing_accounts_label.set_alignment(0.5, 0.0)
         else:
@@ -284,6 +379,15 @@ class Dialog(gtk.Window):
             self.__existing_accounts_label.set_text("Password:")
             self.__existing_accounts_label.set_alignment(0.0, 0.5)    
 
+    def __on_edited_account_type_changed(self, *args):
+        new_iter = self.__account_types_combo.get_active_iter()
+        _logger.debug("account type changed to %s" % self.__account_types_model.get_value(new_iter, 0))
+        if self.__username_link_connect_id is not None:
+            self.__username_link.disconnect(self.__username_link_connect_id)
+
+        self.__username_link.set_text(self.__user_info_types_by_full_name[self.__account_types_model.get_value(new_iter, 0)] + ":")
+        self.__username_link_connect_id = self.__username_link.connect("clicked", self.__open_page, self.__type_links_by_full_name[self.__account_types_model.get_value(new_iter, 0)])
+
     def __on_edited_account_changed(self, *args):
         new_iter = self.__accounts_combo.get_active_iter()
         _logger.debug("account changed to %s" % self.__model.get_value(new_iter, 0))        
@@ -297,9 +401,6 @@ class Dialog(gtk.Window):
                 # TODO: this will trigger __on_password_entry_changed, make sure that is harmless
                 self.__password_entry.set_text(self.__current_account.GetPassword())
                 self.__check_box.set_active(self.__current_account.GetEnabled())
-                # TODO: set the remove button to be sensitive once we enable removing accounts on server
-                self.__remove_button.set_sensitive(False) 
-                self.__remove_link.set_enabled(True)
                 return
         _logger.error("new edited account was not found in self.__model_tree_iter_by_account")
 
@@ -312,6 +413,12 @@ class Dialog(gtk.Window):
             self.__undo_button.set_sensitive(False)  
             self.__apply_button.set_sensitive(False)  
 
+    def __on_save_online_setting_changed(self, widget):
+       # it's important to use the passed in widget here because we have two checkboxes (one in each tab)
+       # that call this function
+       if widget.get_active() != self.__online_accounts_service.GetSaveOnlineFlag():
+           self.__online_accounts_service.SetSaveOnlineFlag(widget.get_active())
+
     def __on_account_settings_applied(self, widget):
         text = self.__password_entry.get_text()
         enabled = self.__check_box.get_active()
@@ -323,15 +430,70 @@ class Dialog(gtk.Window):
     def __on_account_settings_reset(self, widget):
         self.__password_entry.set_text(self.__current_account.GetPassword())
         self.__check_box.set_active(self.__current_account.GetEnabled())
-  
+
+    def __on_account_update_feedback(self, feedback_header, feedback_message, is_error_flag):
+        message_dialog = None
+        if is_error_flag: 
+            message_dialog = gtk.MessageDialog(parent=self, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format=feedback_header)
+        else:
+            message_dialog = gtk.MessageDialog(parent=self, type=gtk.MESSAGE_INFO, buttons=gtk.BUTTONS_OK, message_format=feedback_header)
+            
+        message_dialog.set_default_response(gtk.BUTTONS_OK)
+        message_dialog.format_secondary_text(feedback_message)
+        message_dialog.run()
+        message_dialog.destroy()
+
+    def account_remove_return_cb(self):
+        pass
+
+    def account_remove_error_cb(self, error):
+        self.__on_account_update_feedback("Could not remove an account", error, True)     
+
     def __on_account_remove_clicked(self, widget):
-        self.__online_accounts_service.RemoveAccount(self.__current_account.GetObjectPath())
+        self.__online_accounts_service.RemoveAccount(self.__current_account.GetObjectPath(), self.account_remove_return_cb, self.account_remove_error_cb)
 
     def __on_new_username_changed(self, widget):
         # in the future can check if the input is of the desired form here
         username_entered = (len(self.__username_entry.get_text().strip()) > 0)  
         self.__add_button.set_sensitive(username_entered)        
 
+    # account_tuple consists of object_path, new_account_flag, feedback_message
+    def account_add_return_cb(self, account_tuple):
+        object_path = account_tuple[0]
+        new_account_flag = account_tuple[1]
+        feedback_message = account_tuple[2]
+        
+        if (feedback_message is not None and len(feedback_message) > 0) or (object_path is None and not new_account_flag):
+            if object_path is None and (not new_account_flag): 
+                # feedback_message is an error
+                self.__on_account_update_feedback("Could not add an account", feedback_message, True)
+            else:
+                # feedback_message is a just there to provide extra info
+                self.__username_entry.set_text("")
+                self.__on_account_update_feedback("Information note", feedback_message, False)
+        else: 
+            self.__username_entry.set_text("")
+            self.__notebook.set_current_page(0)
+                
+        # we set the newly added account to be an active iter even if we are not switching to the existing accounts view
+        # because we are showing a warning 
+        if object_path is not None: 
+            if self.__model_tree_iter_by_account.has_key(object_path):
+                # the key will already exist only if the account already existed (new_account_flag should be False in that case)
+                # we could show a message to the user about it, but it is probably more helpful to just show them that account
+                # in the existing accounts view
+                account_iter = self.__model_tree_iter_by_account[object_path]                
+                self.__accounts_combo.set_active_iter(account_iter)
+            else:
+                # acounts system will emit a signal that will cause __on_account_added to be called again, 
+                # but it's ok to call this from here so that we can switch to the new account in the combo box right away
+                self.__on_account_added(object_path)
+                account_iter = self.__model_tree_iter_by_account[object_path]   
+                self.__accounts_combo.set_active_iter(account_iter)
+
+    def account_add_error_cb(self, error):
+        pass
+
     def __on_new_username_added(self, widget):
         text = self.__username_entry.get_text()
         username_entered = (len(text.strip()) > 0)
@@ -340,22 +502,9 @@ class Dialog(gtk.Window):
             return
         
         current_iter = self.__account_types_combo.get_active_iter()
-        for (account_type, type_full_name) in self.__all_account_types:
+        for (account_type, (type_full_name, user_info_type)) in self.__all_account_types.items():
             if self.__account_types_model.get_value(current_iter, 0) == type_full_name:
-                account_tuple = self.__online_accounts_service.GetOrCreateAccount(account_type, text)
-                self.__username_entry.set_text("")
-                self.__notebook.set_current_page(0)
-                # TODO: Display a special message if the account aready existed (account_tuple[1] is False), also
-                # the key should be self.__model_tree_iter_by_account only in that case
-                if self.__model_tree_iter_by_account.has_key(account_tuple[0]):
-                    account_iter = self.__model_tree_iter_by_account[account_tuple[0]]                
-                    self.__accounts_combo.set_active_iter(account_iter)
-                else:
-                    # acounts system will emit a signal that will cause __on_account_added to be called again, 
-                    # but it's ok to call this from here so that we can switch to the new account in the combo box right away
-                    self.__on_account_added(account_tuple[0])
-                    account_iter = self.__model_tree_iter_by_account[account_tuple[0]]   
-                    self.__accounts_combo.set_active_iter(account_iter)
+                self.__online_accounts_service.GetOrCreateAccount(account_type, text, self.account_add_return_cb, self.account_add_error_cb)
                 return 
                 
         _logger.warn("Did not find an account type that matched the account type in the dropdown %s" % self.__account_types_model.get_value(current_iter, 0))
@@ -386,12 +535,18 @@ class Link(gtk.EventBox):
                         & gtk.gdk.ENTER_NOTIFY_MASK
                         & gtk.gdk.LEAVE_NOTIFY_MASK)
 
+    def set_size_request(self, width, height):
+        self.__label.set_size_request(width, height)
+
     def set_alignment(self, x, y):
         self.__label.set_alignment(x, y)
         
     def set_ellipsize(self, do_ellipsize):
         self.__label.set_ellipsize(do_ellipsize)
 
+    def set_line_wrap(self, do_line_wrap):
+        self.__label.set_line_wrap(do_line_wrap)
+
     def __on_button_press(self, self2, e):
         if e.button == 1 and self.__enabled:
             self.emit("clicked")
@@ -401,17 +556,17 @@ class Link(gtk.EventBox):
     def get_text(self):
         return self.__text
     
-    def set_text(self, text):
+    def set_text(self, text, color="blue"):
         self.__text = text
-        self.set_markup(gobject.markup_escape_text(text))
+        self.set_markup(gobject.markup_escape_text(text), color)
     
     def set_enabled(self, enabled):        
         self.__enabled = enabled
         self.set_markup(gobject.markup_escape_text(self.__text))
 
-    def set_markup(self, text):
+    def set_markup(self, text, color="blue"):
         if  self.__enabled: 
-            self.__label.set_markup('<span foreground="blue">%s</span>' % (text,))
+            self.__label.set_markup('<span foreground="%s">%s</span>' % (color, text,))
         else:
             self.__label.set_markup('<span foreground="#666666">%s</span>' % (text,))
 
@@ -427,23 +582,3 @@ class Link(gtk.EventBox):
         if hand and self.__enabled:
             cursor = gtk.gdk.Cursor(display, gtk.gdk.HAND2)
         self.window.set_cursor(cursor)
-
-def get_baseurl():
-    ## first we prefer the base url from the model we're actually using.
-    ## this happens when we just connect to "org.freedesktop.od.Engine"
-    ## and don't know in advance whether a dogfood or production or whatever
-    ## server instance owns that bus name.
-    ## Note that this is _supposed_ to work offline as well - the od.Engine
-    ## is supposed to have an offline mode.
-    model = DataModel()
-    if model.global_resource:
-        try:
-            return model.global_resource.webBaseUrl
-        except AttributeError:
-            pass
-
-    ## we fall back to a hardcoded URL, since it's probably better
-    ## than crashing and would normally be right in production, but never
-    ## right when testing on dogfood.
-    return "http://online.gnome.org"
-
